BRT = LibStub("AceAddon-3.0"):NewAddon("BonusRollTracker", "AceConsole-3.0", "AceEvent-3.0");

-- enable for debug
--local verbose = true;


local ST = LibStub("ScrollingTable");
local DB;

local ADDON_NAME = "Bonus Roll Tracker";
local ADDON_VERSION = "v1.0.7";

local S_WORLD_BOSS = "World Bosses";
local S_LOOT_COLUMN = "Loot";
local S_BOSS_COLUMN = "Boss";
local S_COIN_COLUMN = "Coin";
local S_SPEC_COLUMN = "Spec";
local S_DATE_COLUMN = "Date";
local S_UNKNOWN = "Unknown";
local S_RESET_CONFIRM = "Are you sure you want to clear all bonus roll history?";
local S_COIN_FILTER = "Coin Filter";

local BRT_DATE_WIDTH = 80;
local BRT_BOSS_WIDTH = 250;
local BRT_COIN_WIDTH = 50;
--local BRT_SPEC_WIDTH = 50;
local BRT_LOOT_WIDTH = 370;
local BRT_MARGIN = 25;

local LFR_COLOR = "|cff1eff00";
local NORM_COLOR = "|cff0070dd";
local HERO_COLOR = "|cffa335ee";
local MYTH_COLOR = "|cff800000";



local TABLE_COLUMN_DATE = 1;
local TABLE_COLUMN_BOSS = 2;
local TABLE_COLUMN_COIN = 3;
local TABLE_COLUMN_LOOT = 4;
local TABLE_COLUMN_COIN_INVISIBLE = 5;		-- coin type, not shown
local TABLE_COLUMN_SPEC_INVISIBLE = 6;		-- spec, not shown



local BOSS_DIFFICULTY =
{
	["3"] = NORM_COLOR.."10N".."|r";		-- 10 Normal
	["4"] = NORM_COLOR.."25N".."|r";		-- 25 Normal
	["5"] = HERO_COLOR.."10H".."|r";		-- 10 Heroic
	["6"] = HERO_COLOR.."25H".."|r";		-- 25 Heroic
	["7"] = LFR_COLOR.."LFR".."|r";		-- Old LFR
	["14"] = NORM_COLOR.."N".."|r";		-- WoD Normal
	["15"] = HERO_COLOR.."H".."|r";		-- WoD Heroic
	["16"] = MYTH_COLOR.."M".."|r";		-- WoD Mythic
	["17"] = LFR_COLOR.."LFR".."|r";		-- WoD LFR
}


local SCROLL_COLS =
{
	{
		["name"] = S_DATE_COLUMN;
	    ["width"] = BRT_DATE_WIDTH,
	    ["align"] = "LEFT",
	    --["color"] = { ["r"] = 0.5, ["g"] = 0.5, ["b"] = 1.0, ["a"] = 1.0 },
		["colorargs"] = nil,
		--["bgcolor"] = { ["r"] = 0.0,  ["g"] = 0.0,  ["b"] = 0.0,  ["a"] = 1.0 },	    
		["DoCellUpdate"] = nil,
	},
	{
		["name"] = S_BOSS_COLUMN,
		["width"] = BRT_BOSS_WIDTH,
		["aligh"] = "LEFT",
	},
	{
		["name"] = S_COIN_COLUMN,
		["width"] = BRT_COIN_WIDTH,
		["align"] = "CENTER",
	},
	{
		["name"] = S_LOOT_COLUMN,
		["width"] = BRT_LOOT_WIDTH,
		["align"] = "LEFT",
	}
};


-- Coin types
local COIN_TYPE_CURRENCY =
{
	697,			-- 1 = Elder Charm of Good Fortune
	752,			-- 2 = Mogu Rune of Fate
	776,			-- 3 = Warforged Seal
	994,			-- 4 = Seal of Tempered Fate
}

local CURRENCY_TO_INDEX =
{
	["697"] = 1,
	["752"] = 2,
	["776"] = 3,
	["994"] = 4,
}

local UNKNOWN_ICON = "|TInterface\\Icons\\Inv_misc_questionmark:16|t";
local LOOT_GOLD_ICON = "|TInterface\\Icons\\Inv_misc_coin_02:16|t";


local baseframe = nil;
local scrolltable = nil;

local function FilterMenuOnClick(self, arg1, arg2, checked)
	if (arg1 ~= nil) then
		local old = DB.profile.filters[arg1];
		DB.profile.filters[arg1] = checked;

		if (old ~= checked and scrolltable ~= nil) then
			scrolltable:DoFilter();
			scrolltable:SortData();
			scrolltable:Refresh();
		end
	end
end

local function FilterMenuInit(self, level)
	local info = UIDropDownMenu_CreateInfo();
	for i, item in pairs(COIN_TYPE_CURRENCY) do
		local text = GetCurrencyInfo(item);
		info = UIDropDownMenu_CreateInfo();
		info.text = text;
		info.value = text;
		info.notCheckable = false;
		info.keepShownOnClick = true;
		info.checked = DB.profile.filters[i];
		info.func = FilterMenuOnClick;
		info.arg1 = i;
		UIDropDownMenu_AddButton(info, level);
	end
end

local function ItemFilter(self, rowdata)
	local coinindex = CURRENCY_TO_INDEX[tostring(rowdata[TABLE_COLUMN_COIN_INVISIBLE])];
	if (coinindex ~= nil and DB.profile.filters[coinindex] == true) then
		return true;
	else
		return false;
	end
end

function BRT:InitUI()
	-- Base UI frame
	baseframe = CreateFrame("Frame", "BRTMainFrame", UIParent);
	baseframe:SetWidth(BRT_DATE_WIDTH + BRT_BOSS_WIDTH + BRT_COIN_WIDTH + BRT_LOOT_WIDTH + 2 * BRT_MARGIN);
	baseframe:SetHeight(430);
	baseframe:SetPoint("CENTER",UIParent,"CENTER",0,0);
	baseframe:EnableMouse(true);
	baseframe:SetResizable();
	baseframe:SetMovable(true);
	baseframe:SetFrameStrata("DIALOG");
	baseframe:SetToplevel(true);
	baseframe:SetBackdrop({
		bgFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 64, edgeSize = 12,
		insets = {left = 2, right = 1, top = 2, bottom = 2}
	});
	baseframe:SetBackdropColor(0,0,0,1);
	baseframe:SetBackdropBorderColor(1,1,1,0.2);
	baseframe:SetScript("OnMouseDown", function() baseframe:StartMoving() end)
	baseframe:SetScript("OnMouseUp", function() baseframe:StopMovingOrSizing() end)

	-- Title bar
	local titleframe = CreateFrame("Frame", nil, baseframe)
	titleframe:SetBackdrop({
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
		tile = true, tileSize = 64, edgeSize = 0,
		insets = { left = 0, right = 0, top = 0, bottom = 0 }
	})
	titleframe:SetBackdropColor(0,0,0,1)
	titleframe:SetHeight(22)
	titleframe:EnableMouse()
	titleframe:SetResizable()
	titleframe:SetMovable(true)
	titleframe:SetPoint("LEFT", baseframe, "TOPLEFT", 10, -15)
	titleframe:SetPoint("RIGHT", baseframe, "TOPRIGHT", -10, -15)

	local titletext = titleframe:CreateFontString(nil,"OVERLAY","GameFontNormalSmall")
	titletext:SetPoint("CENTER", titleframe, "CENTER", 0, 1)
	titletext:SetText(string.format("BonusRollTracker %s", ADDON_VERSION ) )
	baseframe.titleframe = titleframe
	--


	-- Close button
	local closebutton = CreateFrame("Button", "CloseButton", baseframe, "UIPanelCloseButton")
	closebutton:SetWidth(32)
	closebutton:SetHeight(32)
	closebutton:SetPoint("TOPRIGHT", baseframe, "TOPRIGHT", -4, 1)
	closebutton:RegisterForClicks("AnyUp")
	closebutton:SetScript("OnClick", function() BRT:OnClickClose() end);


	-- Reset History button
	local resetbutton = CreateFrame("Button", "ResetButton", baseframe, "UIPanelButtonTemplate")
	resetbutton:SetWidth(150)
	resetbutton:SetHeight(25)
	resetbutton:SetPoint("BOTTOMLEFT", baseframe, "BOTTOMLEFT", 8, 8)
	resetbutton:SetText("Clear History")
	resetbutton:RegisterForClicks("AnyUp")
	resetbutton:SetScript("OnClick", function() BRT:OnClickReset() end);


	-- Scrolling Table
	scrolltable = ST:CreateST(SCROLL_COLS, 20, 15, nil, baseframe);
	scrolltable:EnableSelection(false);
	scrolltable.frame:SetPoint("CENTER", baseframe, "CENTER", 0, -16);
	baseframe.scrolltable = scrolltable;

	scrolltable:RegisterEvents({["OnClick"] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, ...)
								BRT:OnItemClick(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, ...)
								end});

	scrolltable:RegisterEvents({["OnEnter"] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, ...)
								BRT:OnItemEnter(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, ...)
								end});

	scrolltable:RegisterEvents({["OnLeave"] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, ...)
								BRT:OnItemLeave(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, ...)
								end});

	scrolltable:SetFilter(ItemFilter);


	-- Filter Dropdown
	CreateFrame("Button", "FilterDropdown", baseframe, "UIDropDownMenuTemplate");
	FilterDropdown:SetPoint("TOPRIGHT", baseframe, "TOPRIGHT", 6, -32);

	UIDropDownMenu_Initialize(FilterDropdown, FilterMenuInit);
	UIDropDownMenu_SetWidth(FilterDropdown, 200);
	UIDropDownMenu_SetText(FilterDropdown, S_COIN_FILTER);


	-- Reset confirm dialog
	StaticPopupDialogs["BRT_CONFIRM_RESET"] = 
	{
		text = S_RESET_CONFIRM,
		button1 = "Yes",
		button2 = "No",
		OnAccept = function() BRT:ConfirmReset() end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		preferredIndex = 3,  -- avoid some UI taint
	}
end

BRT:RegisterChatCommand("brt", "SlashProcessor");

function BRT:OnInitialize()
	DB = LibStub("AceDB-3.0"):New("BonusRollTrackerDB");

	if not DB.char.history then
		DB.char.history = {};
	end

	if not DB.profile.filters then
		DB.profile.filters = { true, true, true, true };
	end

	BRT:InitUI();

	BRT:Populate();
end

function BRT:OnEnable()
end

function BRT:OnDisable()
end


local latestEncounterName = nil;
local latestEncounterID = nil;
local latestEncounterDiff = nil;
local coinType = nil;
local bossName = nil;
local bossDiff = nil;

local tabledata = {};

function BRT:Populate()
	if scrolltable then
		tabledata = {};
		for i, rows in ipairs(DB.char.history) do			
			local col_date = rows[1];
			local col_boss = rows[3];
			local col_loot = UNKNOWN_ICON;
			local col_coin = UNKNOWN_ICON;
			local col_spec = UNKNOWN_ICON;

			-- loot
			local itemname,_,_,_,_,_,_,_,_,itemtex = GetItemInfo(rows[5]);			-- check if loot is an item
			if (itemname ~= nil) then
				col_loot = "|T"..itemtex..":16|t".." "..rows[5];
			else
				col_loot = LOOT_GOLD_ICON.." "..rows[5];
			end

			-- coin icon
			local coinname,_,cointex = GetCurrencyInfo(rows[2]);
			if (coinname ~= nil) then
				col_coin = "|T"..cointex..":16|t";
			end

			-- spec icon
		--	local _,specname,_,spectex = GetSpecializationInfoByID(rows[6]);
		--	if (specname ~= nil) then
		--		col_spec = "|T"..spectex..":16|t";
		--	end

			-- concatenate boss difficulty (if applicable)
			local difficulty = BOSS_DIFFICULTY[tostring(rows[4])];
			if (difficulty ~= nil) then
				col_boss = col_boss .. " (" .. difficulty .. ")";
			end

			local fe = {col_date, col_boss, col_coin, col_loot, rows[2]};
			table.insert(tabledata, 1, fe);
		end
		scrolltable:SetData(tabledata, true);
	end
end

function BRT:Loot(timeStamp, coinType, bossName, bossDiff, lootName, lootSpec)
	if verbose then
		BRT:Print(timeStamp .. ": Bonus rolled " .. bossName .. " with " .. coinType .. ", looted " .. lootName);
	end

	local entry = {timeStamp, coinType, bossName, bossDiff, lootName, lootSpec};
	table.insert(DB.char.history, entry);

	-- format DB data into Scrolling Table
	BRT:Populate();
end

function BRT:BONUS_ROLL_RESULT(event, ...)
	local rewardType, rewardLink, rewardQuantity, rewardSpecID = ...;

	if verbose then
		BRT:Print("Bonus roll: type " .. rewardType .. ", link " .. rewardLink .. ", " .. rewardQuantity);
	end

	local lootName = nil;
	local lootSpec = GetLootSpecialization();		--rewardSpecID is wrong if reward is gold

	if (rewardType == "money") then
		lootName = GetCoinText(rewardQuantity);
	elseif (rewardType == "item") then
		lootName = rewardLink;
	end

	if (bossName == nil) then bossName = S_UNKNOWN; end

	if (lootName ~= nil) then
		local timestamp = date("%x");
		BRT:Loot(timestamp, coinType, bossName, bossDiff, lootName, lootSpec);
	end

	latestEncounterID = nil;
	latestEncounterName = nil;
	latestEncounterDiff = nil;
end

function BRT:ENCOUNTER_START(event, ...)
	latestEncounterID = nil;
	latestEncounterName = nil;
	latestEncounterDiff = nil;
end

function BRT:ENCOUNTER_END(event, ...)
	local encounterID, encounterName, difficultyID, raidSize, endStatus = ...
	if verbose then
		BRT:Print("Encounter end: id = " .. encounterID .. ", name = " .. encounterName .. ", difficulty = " .. difficultyID);
	end

	if (endStatus == 1) then
		latestEncounterName = encounterName;
		latestEncounterID = encounterID;
		latestEncounterDiff = difficultyID;
	end
end

function BRT:BONUS_ROLL_ACTIVATE(event, ...)
	-- get coin type from BonusRollFrame
	coinType = BonusRollFrame.currencyID;

	-- get boss name
	if (IsInInstance() == true) then
		-- if we're inside an instance, use the encounter name
		bossName = latestEncounterName;
		bossDiff = latestEncounterDiff;
	else
		-- not an instance, let's just call it a World Boss
		bossName = S_WORLD_BOSS;
		bossDiff = nil;
	end
end

function BRT:SlashProcessor(input)
	if (baseframe:IsShown()) then
		baseframe:Hide();
	else
		baseframe:Show();
	end
end

function BRT:OnItemClick(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, ...)

end

function BRT:OnItemEnter(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, ...)
	if (column == TABLE_COLUMN_LOOT) then		-- loot
		if (data[realrow] ~= nil) then
			local loot = data[realrow][TABLE_COLUMN_LOOT];
			if (loot ~= nil) then
				local itemname = GetItemInfo(loot);		-- make sure we have a valid item for tooltip
				if (itemname ~= nil) then
					GameTooltip:SetOwner(baseframe, "ANCHOR_TOPRIGHT");
					GameTooltip:SetHyperlink(loot);					
				end
			end
		end
	elseif (column == TABLE_COLUMN_COIN) then	-- coin
		if (data[realrow] ~= nil) then
			local coin = data[realrow][TABLE_COLUMN_COIN_INVISIBLE];
			if (coin ~= nil) then
				local coinlink = GetCurrencyLink(coin);
				if (coinlink ~= nil) then
					GameTooltip:SetOwner(baseframe, "ANCHOR_TOPRIGHT");
					GameTooltip:SetHyperlink(coinlink);
				end
			end
		end
	end
end

function BRT:OnItemLeave(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, ...)
	if (column == TABLE_COLUMN_COIN or column == TABLE_COLUMN_LOOT) then
		if (GameTooltip:IsOwned(baseframe)) then
			GameTooltip:Hide();
		end
	end
end

function BRT:OnClickClose()
	baseframe:Hide();

	if (GameTooltip:IsOwned(baseframe)) then
		GameTooltip:Hide();
	end
end

function BRT:OnClickReset()
	StaticPopup_Show("BRT_CONFIRM_RESET");
end

function BRT:ConfirmReset()
	DB.char.history = {};

	BRT:Populate();
end

BRT:RegisterEvent("BONUS_ROLL_RESULT");
BRT:RegisterEvent("BONUS_ROLL_ACTIVATE");
BRT:RegisterEvent("ENCOUNTER_END");
BRT:RegisterEvent("ENCOUNTER_START");

-- allow ESC to close our frame
tinsert(UISpecialFrames, "BRTMainFrame");